@@ -218,7 +218,8 @@ import migrate.changeset
 from sqlalchemy import orm
 
 from sqlalchemy import create_engine  , MetaData
-from sqlalchemy import Table, Column, Integer, String, Float,  Text, UnicodeText, LargeBinary, DateTime, FLOAT
+from sqlalchemy import Table, Column, Integer, String, Float,  Text, UnicodeText, LargeBinary, DateTime, PickleType, Boolean
+from sqlalchemy import BLOB
 from sqlalchemy import ForeignKey
 from sqlalchemy import Index
 from sqlalchemy import event
@@ -259,15 +260,19 @@ python_to_sa_conversion = {
                                                         int : Integer,
                                                         datetime : DateTime,
                                                         float : Float,
+                                                        #~ object: PickleType,
+                                                        buffer: LargeBinary,
+                                                        bool  : Boolean,
                                                         }
 #~ sa_to_python_conversion = { }
 #~ for k,v in python_to_sa_conversion.items():
     #~ sa_to_python_conversion[v] = k
 
 
-global globalsesession
-globalsesession = None
-
+global globalsession
+globalsession = None
+global globaldbinfo
+globaldbinfo = None
 
 
 
@@ -304,7 +309,10 @@ def create_column_if_not_exists(table, attrname, attrtype):
             col.create( table)
     elif attrtype in python_to_sa_conversion:
         if attrname not in colnames:
-            col = Column(attrname, python_to_sa_conversion[attrtype])
+            if attrtype is buffer:
+                col = Column(attrname, LargeBinary(MAX_BINARY_SIZE))
+            else:
+                col = Column(attrname, python_to_sa_conversion[attrtype])
             col.create( table )
     else:
         raise NotImplementedError
@@ -528,8 +536,14 @@ def create_classes_from_schema_sniffing( engine, oeclasses,
                 # so we must go up in class inheritance
                 t = None
                 for ptype, satype in python_to_sa_conversion.items():
+                    #~ print col.name, ptype, satype, col.type, type(col.type), issubclass(type(col.type), satype)
                     if issubclass(type(col.type), satype):
                         t = ptype
+                #~ if t is None and type(col.type) == BLOB:
+                    #~ t = object
+                    #~ col.type = PickleType
+                    #~ print col.type
+                #~ print col.name, t
                 genclass.usable_attributes[col.name] = t
         generated_classes.append(genclass)
 
@@ -978,7 +992,9 @@ class  MyBasicCache():
         #~ print 'cache size', len(self.d)
         while len(self.d) >= self.maxsize:
             self.d.popleft()
-        
+    
+    def clear(self):
+        self.d.clear()
 
 
 class EventLoadListennerForCache:
@@ -1010,6 +1026,14 @@ class DataBaseConnectionInfo(object):
     """
     def __init__(self, **kargs):
         self.__dict__.update(kargs)
+        
+        self.classes_by_name = dict([ (c.__name__, c) for c in self.mapped_classes])
+    
+    def get_class(self, name):
+        return self.classes_by_name.get(name, None)
+    
+        
+        
 
 
 def open_db(url, myglobals = None, suffix_for_class_name = '', use_global_session = True, 
@@ -1100,13 +1124,16 @@ def open_db(url, myglobals = None, suffix_for_class_name = '', use_global_sessio
     Session = orm.scoped_session(orm.sessionmaker(bind=metadata.bind , autocommit=False, autoflush=True))
     #~ Session = orm.sessionmaker(bind=metadata.bind , autocommit=False, autoflush=True)
     
-    if use_global_session:
-        global globalsesession
-        globalsesession = Session()
     
     dbinfo = DataBaseConnectionInfo( url =url,mapped_classes = generated_classes,Session = Session,
                                                 metadata = metadata, cache = cache, numpy_storage_engine = numpy_storage_engine,
                                                 compress = compress)
+
+    if use_global_session:
+        global globalsession
+        globalsession = Session()
+        global globaldbinfo
+        globaldbinfo = dbinfo
     
     return dbinfo
     
@@ -1129,7 +1156,7 @@ def execute_sql(query ,session = None, column_split = True,
     
     """
     if session is None:
-        session = globalsesession
+        session = globalsession
     assert session is not None, 'You must give a session for execute_sql'
     
     pres = session.execute(query, kargs)
