@@ -3,12 +3,13 @@
 Some QT widget for viewing neo object: AnalogSignal, EventArray, EpochArray, SpikeTrain
 
 """
-from .tools import TimeSeeker, XSizeChanger, YLimsChanger
+from .tools import TimeSeeker
 from .signalviewer import SignalViewer
 from .epochviewer import EpochViewer
+from .eventviewer import EventViewer
 from .segmentviewer import SegmentViewer
 from .timefreqviewer import TimeFreqViewer
-
+from .spiketrainviewer import SpikeTrainViewer
 
 
 
